@@ -3,12 +3,12 @@
 #     /  \		(C) Copyright 2003 Parliament Hill Computers Ltd.
 #     \  /		All rights reserved.
 #      \/
-#       .		Author: Alain Williams, January 2003
+#       .		Author: Alain Williams, First written January 2003; updated March 2015
 #       .		addw@phcomp.co.uk
 #        .
 #          .
 #
-#	SCCS: @(#)Expression.pm	1.21 03/04/08 09:02:54
+#	SCCS: @(#)Expression.pm	1.36 03/27/15 13:11:31
 #
 # This module is free software; you can redistribute it and/or modify
 # it under the same terms as Perl itself. You must preserve this entire copyright
@@ -43,67 +43,106 @@ use vars qw/
 	&FuncValue
 	&Parse
 	&ParseString
+	&ParseToScalar
 	&SetOpts
 	&VarSetFun
 	&VarSetScalar
 	$Version
 );
 
-our $VERSION = "1.21";
+our $VERSION = "1.36";
+
+# Fundamental to this is a tree of nodes.
+# Nodes are hashes with members:
+# oper (var, *, >, ...)
+# left & right (refs to nodes)
+# monop (boolean)
+# name (on var nodes)
+# fname (on func nodes)
+# val (on const nodes)
+# flow (on flow nodes)
+
 
 # Operator precedence, higher means evaluate first.
 # If precedence values are the same associate to the left.
 # 2 values, depending on if it is the TopOfStack or JustRead operator - [TOS, JR]. See ':=' which right associates.
 # Just binary operators makes life easier as well.
-my $HighestOperPrec = 17;
-my $PrecTerminal = 18;		# Precedence of terminal (or list) - ie operand
+# Getting the precedence values right is a pain and for things like close paren, non obvious.
+
 my %OperPrec = (
-	'('	=>	[19, 19],
-	'var'	=>	[18, 18],
-	'const'	=>	[18, 18],
-	'func'	=>	[18, 18],
-	'M-'	=>	[16, 17],	# Monadic -
-	'M+'	=>	[16, 17],	# Monadic +
-	'M!'	=>	[16, 17],
-	'M~'	=>	[16, 17],
-	'**'	=>	[15, 15],
-	'*'	=>	[14, 14],
-	'/'	=>	[14, 14],
-	'%'	=>	[14, 14],
-	'+'	=>	[13, 13],
-	'-'	=>	[13, 13],
-	'.'	=>	[12, 12],
-	'>'	=>	[11, 11],
-	'<'	=>	[11, 11],
-	'>='	=>	[11, 11],
-	'<='	=>	[11, 11],
-	'=='	=>	[11, 11],
-	'!='	=>	[11, 11],
-	'<>'	=>	[11, 11],
-	'lt'	=>	[11, 11],
-	'gt'	=>	[11, 11],
-	'le'	=>	[11, 11],
-	'ge'	=>	[11, 11],
-	'eq'	=>	[11, 11],
-	'ne'	=>	[11, 11],
-	'&&'	=>	[10, 10],
-	'||'	=>	[9, 9],
-	':'	=>	[8, 8],
-	'?'	=>	[7, 7],
-	','	=>	[6, 6],
-	':='	=>	[4, 5],
-	')'	=>	[3, 3],
-#	';'	=>	[0, 0],
+	'var'	=>	[23, 23],
+	'const'	=>	[23, 23],
+	'['	=>	[7, 22],
+	'++'	=>	[22, 22],
+	'--'	=>	[22, 22],
+	'M-'	=>	[20, 21],	# Monadic -
+	'M+'	=>	[20, 21],	# Monadic +
+	'M!'	=>	[20, 21],
+	'M~'	=>	[20, 21],
+	'**'	=>	[19, 19],
+	'*'	=>	[18, 18],
+	'/'	=>	[18, 18],
+	'%'	=>	[18, 18],
+	'+'	=>	[17, 17],
+	'-'	=>	[17, 17],
+	'.'	=>	[16, 16],
+	'>'	=>	[15, 15],
+	'<'	=>	[15, 15],
+	'>='	=>	[15, 15],
+	'<='	=>	[15, 15],
+	'=='	=>	[15, 15],
+	'!='	=>	[15, 15],
+	'<>'	=>	[15, 15],
+	'lt'	=>	[15, 15],
+	'gt'	=>	[15, 15],
+	'le'	=>	[15, 15],
+	'ge'	=>	[15, 15],
+	'eq'	=>	[15, 15],
+	'ne'	=>	[15, 15],
+	'&&'	=>	[14, 14],
+	'||'	=>	[13, 13],
+	':'	=>	[12, 12],
+	'?'	=>	[11, 11],
+	','	=>	[10, 10],
+	'('	=>	[9, 22],
+	')'	=>	[9, 9],	
+	'func'	=>	[21, 22],
+	']'	=>	[7, 7],
+	':='	=>	[5, 6],		# 6 to make := right assosc
+	';'	=>	[2, 2],
+	'{'	=>	[1, 0],
+	'}'	=>	[4, 0],
+	'flow'	=>	[3, 4],
+	'EOF'	=>	[-50, -50],
 );
+# TOS, JR
+# Nothing special about -ve precedence, just saves renumbering when I got to zero.
 
 # Monadic/Unary operators:
 my %MonOp = (
-	'-'	=>	16,
-	'+'	=>	16,
-	'!'	=>	16,
-	'~'	=>	16,
+	'-'	=>	20,
+	'+'	=>	20,
+	'!'	=>	20,
+	'~'	=>	20,
 );
 
+# MonVarOp - operate on variables, but treat much like monops:
+my %MonVarOp = (
+	'++'	=>	22,
+	'--'	=>	22,
+);
+
+# Closing operators on opening ones. NOT [ ]
+my %MatchOp = (
+	'('	=>	')',
+	'{'	=>	'}',
+);
+
+my %MatchOpClose = reverse %MatchOp; # Reverse lookup
+
+# Inbuilt functions
+my %InFuns = map { $_ => 1} qw/ int abs round split join printf mktime strftime localtime defined aindex /;
+
 # Default error output function
 sub PrintError {
 	printf STDERR @_;
@@ -113,16 +152,34 @@ sub PrintError {
 # Default function to set a variable value, store as a reference to an array.
 # Assign to a variable. (Default function.) Args:
 # 0	Self
-# 1	Variable name
+# 1	Variable name, might look like a[2] in which case set element with last value in arg 2
+#	Don't make an array bigger it already is, except to make it 1 element bigger
 # 2	Value - an array
 # Return the value;
 sub VarSetFun {
 	my ($self, $name, @value) = @_;
 
 	unless(defined($name)) {
-		$self->{PrintErrFunc}("Undefined variable name - need () to force left to right assignment ?");
+		$self->{PrintErrFunc}("Undefined variable name '$name' - need () to force left to right assignment ?");
 	} else {
-		$self->{VarHash}->{$name} = \@value;
+		if($name =~ /^(.+)\[(\d+)\]$/) {
+			unless(defined($self->{VarHash}->{$1})) {
+				if($2 == 0) {
+					$self->{VarHash}->{$1} = $value[-1];
+				} else {
+					$self->{PrintErrFunc}("Can only create variable '%s' by setting element 0", $1);
+				}
+			} elsif($2 > $self->{ArrayMaxIndex}) {
+				$self->{PrintErrFunc}("Array index %d is too large. Max is %d", $2, $self->{ArrayMaxIndex});
+			} elsif($2 > @{$self->{VarHash}->{$1}}) {
+				$self->{PrintErrFunc}("Extending array too much, '%s' has %d elements, trying to set element %d", $1, scalar @{$self->{VarHash}->{$1}}, $2);
+		
+			} else {
+				$self->{VarHash}->{$1}[$2] = $value[-1];
+			}
+		} else {
+			$self->{VarHash}->{$name} = \@value;
+		}
 	}
 
 	return @value;
@@ -167,94 +224,130 @@ sub VarIsDefFun {
 sub ParseString {
 	my ($self, $expr) = @_;
 
-	my %top = (oper	=> '(');
-	my @operators = (\%top);	# Put '(' at top of the tree, matches virtual Ket
-	my @operands;			# Parsed tree ends up here
-	my $nodep;
+	my @operators = ();		# Operators stacked here until needed
+	my @tree;			# Parsed tree ends up here
+	my $newt;			# New Token
 
-	my $operlast = 1;		# Start with a virtual '(', rework if we have postfix operators
+	my $operlast = 1;		# Operator was last, ie not: var, const, ; ) string flow. Used to idenify monadic operators
+	my $endAlready = 0;
+	my $GenSemiColon = 0;		# Need to generate a ';'. Always do so after a '}'
 
 	while(1) {
-		$nodep = {()};
+		my $semi = 0;
+		$newt = {};
 
 		# Lexical part:
 
 		$expr =~ s/^\s*//;
-		my $VirtKet = $expr eq '';
+		my $EndInput = $expr eq '';
 
-		# Match integer/float constant:
-		if($expr =~ s/^(((\d+(\.\d*)?)|(\.\d+))([ed][-+]?\d+)?)//i) {
-			$nodep->{oper} = 'const';
-			$nodep->{val} = $1;
+		if($GenSemiColon) {
+			# Generate an extra semicolon - after a close brace
+			$newt->{oper} = ';';
+			$operlast = 0;
+			$EndInput = $GenSemiColon = 0;
+		} # End of input string:
+		elsif($EndInput) {
+			$operlast = 0;
+			# First time generate a ';' to terminate a set of statements:
+			if($endAlready) {
+				undef $newt;
+			} else {
+				$newt->{oper} = 'EOF';
+				$EndInput = 0;
+			}
+			$endAlready = 1;
+		} # Match integer/float constant:
+		elsif($expr =~ s/^(((\d+(\.\d*)?)|(\.\d+))([ed][-+]?\d+)?)//i) {
+			$newt->{oper} = 'const';
+			$newt->{val} = $1;
 			$operlast = 0;
 		} # Match string bounded by ' or "
 		elsif($expr =~ /^(['"])/ and $expr =~ s/^$1([^$1]*)$1//) {
-			$nodep->{oper} = 'const';
-			$nodep->{val} = $1;
+			$newt->{oper} = 'const';
+			$newt->{val} = $1;
 			$operlast = 0;
-		} # Match (operators)
-		elsif($expr =~ s@^(:=|>=|<=|==|<>|!=|&&|\|\||lt|gt|le|ge|eq|ne|\*\*|[-~!./*%+,<>\?:\(\);])@@) {
-			$nodep->{oper} = $1;
-			# Monadic if the previous token was an operator and this one can be mondic:
+		} elsif($expr =~ s/^}//) {
+			# Always need a ';' after this - magic one up to be sure
+			# If not then flow operators screw up.
+			$newt->{oper} = '}';
+			$GenSemiColon = 1;
+			$operlast = 1;
+		} # Match (operators). Need \b after things like 'ne' so that it is not start of var name:
+		elsif($expr =~ s@^(\+\+|--|:=|>=|<=|==|<>|!=|&&|\|\||lt\b|gt\b|le\b|ge\b|eq\b|ne\b|\*\*|[-~!./*%+,<>\?:\(\)\[\]{])@@) {
+			$newt->{oper} = $1;
+			# Monadic if the previous token was an operator and this one can be monadic:
 			if($operlast && defined($MonOp{$1})) {
-				$nodep->{oper} = 'M' . $1;
-				$nodep->{monop} = $1;		# Monop flag & for error reporting
+				$newt->{oper} = 'M' . $1;
+				$newt->{monop} = $1;		# Monop flag & for error reporting
+			}
+			if(defined($MonVarOp{$1})) {
+				$newt->{monop} = $1;		# Monop flag & for error reporting
 			}
 
-			$operlast = 1 unless($1 eq ')');
+			$operlast = 1 unless($1 eq ')' or $1 eq ']');
 
-		} # End of input string:
-		elsif($VirtKet) {
-			$nodep->{oper} = ')';
+		} # Flow: if/while:
+		elsif($expr =~ s@^(if|while)@@) {
+			$newt->{oper} = 'flow';
+			$newt->{flow} = $1;
+			$operlast = 0;
+		} # Semi-colon:
+		elsif($expr =~ s@^;@@) {
+			$newt->{oper} = ';';
+			$operlast = 0;
 		} # Match 'function(', leave '(' in input:
-		elsif($expr =~ s/^([a-zA-Z][\w]*)\(/(/) {
-			$nodep->{oper} = 'func';
-			$nodep->{fname} = $1;
+		elsif($expr =~ s/^([_a-zA-Z][\w]*)\(/(/) {
+			unless($InFuns{$1}) {
+				$self->{PrintErrFunc}("Unknown function '%s'", $1);
+				return;
+			}
+			$newt->{oper} = 'func';
+			$newt->{fname} = $1;
+			$operlast = 1;    # So that argument can be monadic
 		} # Match ${SomeNonWhiteCharsOrCurlies} or VarName or $VarName or $123 or $OneNonSpaceCharacter
 		elsif($expr =~ s/^\$\{([^\s{}]+)\}|^\$(\d+|[_a-zA-Z]\w*|[^\s])|^([_a-zA-Z]\w*)//) {
-			$nodep->{oper} = 'var';
-			$nodep->{name} = defined($1) ? $1 : defined($2) ? $2 : $3;
+			$newt->{oper} = 'var';
+			$newt->{name} = defined($1) ? $1 : defined($2) ? $2 : $3;
 			$operlast = 0;
 		} else {
 			$self->{PrintErrFunc}("Unrecognised input in expression at '%s'", $expr);
 			return;
 		}
 
-		# Grammatical part:
+		# Processed everything ?
+		if(!@operators && $EndInput) {
+			return pop @tree;
+		}
 
-		while(1) {
-			my $NewOpPrec = $OperPrec{$nodep->{oper}}[1];
+		# Grammatical part
+		# Move what we can from @operators to @tree
+
+		my $loopb = 0; # Loop buster
+		while(@operators || $newt) {
 
 			# End of input ?
-			if($VirtKet and $#operators == 0) {
-				if($#operands != 0) {
+			if($EndInput and @operators == 0) {
+				if(@tree != 1) {	# There should be one node left - the root
 					$self->{PrintErrFunc}("Expression error - %s",
-						$#operands == -1 ? "it's incomplete" : "missing operator");
+						$#tree == -1 ? "it's incomplete" : "missing operator");
 					return;
 				}
-				return pop @operands;
+				return pop @tree;
 			}
 
-			# Terminal (var/const) - KV not '(' or 'func':
-			if($NewOpPrec >= $PrecTerminal and $nodep->{oper} ne '(' and $nodep->{oper} ne 'func') {
-				push @operands, $nodep;
+			# Terminal (var/const). Shift: push it onto the tree:
+			if($newt and ($newt->{oper} eq 'var' or $newt->{oper} eq 'const')) {
+				$operators[-1]->{after} = 1 if(@operators);
+
+				push @tree, $newt;
 				last;	# get next token
 			} # It must be an operator, which must have a terminal to it's left side:
 
-			if($#operators < 0) {
-				$self->{PrintErrFunc}("Syntax error at '%s'", $expr);
-				return;
-			}
-
-			# Eliminate ()
-			if($operators[$#operators]->{oper} eq '(' and $nodep->{oper} eq ')') {
-				if($VirtKet and $#operators != 0) {
-					$self->{PrintErrFunc}("Unexpected end of expression with unmatched '('");
-					return;
-				}
-
-				if(!$VirtKet and $#operators == 0) {
-					$self->{PrintErrFunc}("Unexpected ')'");
+			# Eliminate () - where current node is a close bracket
+			if($newt and @operators and $operators[-1]->{oper} eq '(' and $newt->{oper} eq ')') {
+				if($EndInput and $#operators != 0) {
+					$self->{PrintErrFunc}("Unexpected end of expression with unmatched '$operators[-1]->{oper}'");
 					return;
 				}
 
@@ -262,30 +355,86 @@ sub ParseString {
 				last;	# get next token
 			}
 
-			# New operator binds more than top op, push so that we get the RH expression:
-			if($NewOpPrec > $OperPrec{$operators[$#operators]->{oper}}[0] or $operators[$#operators]->{oper} eq '(') {
-				push @operators, $nodep;
+			# Should have a new node to play with - unless end of string
+			if(!$newt && !$EndInput) {
+				if($loopb++ > 40) {
+					$self->{PrintErrFunc}("Internal error, infinite loop at: $expr");
+					return;
+				}
+				next;
+			}
+
+			my $NewOpPrec;	# EOF is ultra low precedence
+			$NewOpPrec = ($newt) ? $OperPrec{$newt->{oper}}[1] : -100; # Just read precedence
+				
+
+			# If there is a new operator & it is higher precedence than the one at the top of @operators, push it
+			# Also put if @operators is empty
+			if($newt && @operators) {
+				print "Undefined NEWOPrec\n" unless defined $NewOpPrec;
+				print "undefeined op-1 oper '$operators[-1]->{oper}'\n" unless(defined $OperPrec{$operators[-1]->{oper}}[0]);
+			}
+			if($newt && (!@operators or (@operators && $NewOpPrec > $OperPrec{$operators[-1]->{oper}}[0]))) {
+				$operators[-1]->{after} = 1 if(@operators);
+				push @operators, $newt;
 				last;	# get next token
 			}
 
-			# Reduce: While we have LH & RH operands, replace top 2 operands with node that evaluates them:
-			while($NewOpPrec <= $OperPrec{$operators[$#operators]->{oper}}[0] and $operators[$#operators]->{oper} ne '(') {
-				my $top = pop @operators;
-				my $func = $top->{oper} eq 'func';
+			# Flows (if/while) must not be reduced unless the newop is ';' '}' 'EOF' - ALSO PUSH
+			if(@operators && $operators[-1]->{oper} eq 'flow' && $newt && $newt->{oper} ne ';' && $newt->{oper} ne 'EOF' && $newt->{oper} ne '}') {
+				$operators[-1]->{after} = 1 if(@operators);
+				push @operators, $newt;
+				last;
+			}
 
-				my $monop = defined($top->{monop});
+			# Reduce, ie where we have everything move operators from @operators to @tree, their operands will be on @tree
+			# Reduce when the new operator precedence is lower than or equal to the one at the top of @operators
+			if(@operators && $NewOpPrec <= $OperPrec{$operators[-1]->{oper}}[0]) {
 
-				unless($#operands >= (($func | $monop) ? 0 : 1)) {
-					$self->{PrintErrFunc}("Missing operand to operator '%s' at %s", $top->{oper},
+				# One of the pains is a trailing ';', ie nothing following it.
+				# Detect it and junk it
+				if($operators[-1]->{oper} eq ';' && !defined $operators[-1]->{after}) {
+					pop @operators;
+					next;
+				}
+
+				# If top op is { & new op is } - pop them:
+				if(@operators && $newt && $operators[-1]->{oper} eq '{' && $newt->{oper} eq '}') {
+					pop @operators; # Lose the open curly
+
+					# Unless we uncovered a flow - get next token
+					last unless(@operators && $operators[-1]->{oper} eq 'flow');
+
+					$newt = undef; # So that we do a last below
+				}
+				my $op = pop @operators;
+				my $func = $op->{oper} eq 'func';
+				my $flow = $op->{oper} eq 'flow';
+				my $monop = defined($op->{monop});
+
+				# Enough on the tree ?
+				unless(@tree >= (($func | $monop | $flow) ? 1 : 2)) {
+					# ';' are special, don't need operands, also can lose empty ';' nodes
+					next if($op->{oper} eq ';' or $op->{oper} eq 'EOF');
+
+					$self->{PrintErrFunc}("Missing operand to operator '%s' at %s", $op->{oper},
 						($expr ne '' ? "'$expr'" : 'end'));
+
 					return;
 				}
 
-				# With 2 operands we can treat as an operand:
-				$top->{right} = pop @operands;
-				$top->{left}  = pop @operands unless($func or $monop);
+				$op->{right} = pop @tree;
+				unless($monop or $func) {
+					$op->{left} = pop @tree;
+				}
 
-				push @operands, $top;
+				$op->{oper} = ';' if($op->{oper} eq 'EOF'); # ie join to previous
+				push @tree, $op;
+
+				$newt = undef
+					if($newt && $op->{oper} eq '[' && $newt->{oper} eq ']');
+
+				last unless($newt); # get next token
 			}
 		}
 	}
@@ -311,13 +460,23 @@ sub CheckTreeInt {
 
 	my $ok = 1;
 
+	if(defined($MatchOp{$tree->{oper}}) or defined($MatchOpClose{$tree->{oper}})) {
+		$self->{PrintErrFunc}("Unmatched bracket '%s'", $tree->{oper});
+		$ok = 0;
+	}
+
+	if(defined($MonVarOp{$tree->{oper}}) and (!defined($tree->{right}) or $tree->{right}{oper} ne 'var')) {
+		$self->{PrintErrFunc}("Operand to '%s' must be a variable", $tree->{oper});
+		$ok = 0;
+	}
+
 	if($tree->{oper} eq '?' and $tree->{right}{oper} ne ':') {
 		$self->{PrintErrFunc}("Missing ':' operator after '?' operator") unless($ChkErrs);
 		$ok = 0;
 	}
 
 	if($tree->{oper} ne 'func') {
-		unless((!defined($tree->{left}) and defined($tree->{monop})) or &CheckTree($self, $tree->{left})) {
+		unless((!defined($tree->{left}) and defined($tree->{monop})) or $self->CheckTree($tree->{left})) {
 			$self->{PrintErrFunc}("Missing LH expression to '%s'", defined($tree->{monop}) ? $tree->{monop} : $tree->{oper}) unless($ChkErrs);
 			$ok = 0;
 		}
@@ -338,7 +497,7 @@ sub CheckTreeInt {
 sub Parse {
 	my ($self, $expr) = @_;
 
-	return &CheckTree($self, &ParseString($self, $expr));
+	return $self->CheckTree($self->ParseString($expr));
 }
 
 # Print a tree - for debugging purposes. Args:
@@ -366,12 +525,12 @@ sub PrintTree {
 	} else {
 		print "\n";
 		print "    " x $dl;print "Desc L \n";
-		&PrintTree($self, $nodp->{left}, $dl);
+		$self->PrintTree($nodp->{left}, $dl);
 
 		print "    " x $dl;print "op '$nodp->{oper}' P $OperPrec{$nodp->{oper}}[0] at $nodp\n";
 
 		print "    " x $dl;print "Desc R \n";
-		&PrintTree($self, $nodp->{right}, $dl);
+		$self->PrintTree($nodp->{right}, $dl);
 	}
 }
 
@@ -381,11 +540,24 @@ sub PrintTree {
 # 1	The root of a tree.
 sub EvalToScalar {
 	my ($self, $tree) = @_;
-	my @res = &EvalTree($self, $tree, 0);
+	my @res = $self->Eval($tree);
 
 	return $res[$#res];
 }
 
+# Parse a string, check and evaluate it, return a scalar
+# Args:
+# 0	Self
+# 1	String to evaluate.
+# Return undef on error.
+sub ParseToScalar {
+	my ($self, $expr) = @_;
+
+	my $tree = $self->Parse($expr);
+	return undef unless($tree);
+	return $self->EvalToScalar($tree);
+}
+
 # Evaluate a tree. The result is an array, if you are expecting a single value it is the last (probably $#'th) element.
 # Args:
 # 0	Self
@@ -393,7 +565,10 @@ sub EvalToScalar {
 sub Eval {
 	my ($self, $tree) = @_;
 
-	return &EvalTree($self, $tree, 0);
+	$self->{LoopCount} = 0;	# Count all loops
+	$self->{VarSetFun}($self, '_TIME', time);
+
+	return $self->EvalTree($tree, 0);
 }
 
 # Evaluate a tree. The result is an array, if you are expecting a single value it is the last (probably $#'th) element.
@@ -415,14 +590,14 @@ sub EvalTree {
 
 	# Recognise the 'defined' func specially - it needs a lvalue
 	return $self->{FuncEval}($self, $tree->{fname},
-				&EvalTree($self, $tree->{right}, $tree->{fname} eq 'defined'))		if($oper eq 'func');
+				$self->EvalTree($tree->{right}, $tree->{fname} eq 'defined'))		if($oper eq 'func');
 
 	# Monadic operators:
 	if(!defined($tree->{left}) and defined($tree->{monop})) {
 		$oper = $tree->{monop};
 
 		# Evaluate the (RH) operand
-		my @right = &EvalTree($self, $tree->{right}, 0);
+		my @right = $self->EvalTree($tree->{right}, 0);
 		my $right = $right[$#right];
 		unless(defined($right)) {
 			unless($self->{AutoInit}) {
@@ -432,7 +607,7 @@ sub EvalTree {
 			$right = 0;	# Monadics are all numeric
 		}
 
-		unless($right =~ /^([-+]?)0*([\d.]+)([ef][-+]?\d*|)/i) {
+		unless($right =~ /^([-+]?)0*([\d.]+)([ef][-+]?\d*|)$/i) {
 			$self->{PrintErrFunc}("Operand to monadic '%s' is not numeric '%s'", $oper, $right);
 			return;
 		}
@@ -442,30 +617,71 @@ sub EvalTree {
 		return  $right if($oper eq '+');
 		return !$right if($oper eq '!');
 		return ~$right if($oper eq '~');
+		if($oper eq '++' or $oper eq '--') {
+			$oper eq '++' ? $right++ : $right--;
+
+			return $self->{VarSetFun}($self, $tree->{right}{name}, $right);
+		}
+
+		$self->{PrintErrFunc}("Unknown monadic operator when evaluating: '%s'", $oper);
+		return;
 	}
 
 	# This is complicated by multiple assignment: (a, b, c) := (1, 2, 3, 4). 'c' is given '(3, 4)'.
+	# Assign the right value to the left node
+	# Where the values list is shorter, leave vars alone: (a, b, c) := (1, 2) does not change c.
 	if($oper eq ':=') {
-		my @left = &EvalTree($self, $tree->{left}, 1);
-		my @right = &EvalTree($self, $tree->{right}, $wantlv);
+		my @left = $self->EvalTree($tree->{left}, 1);
+		my @right = $self->EvalTree($tree->{right}, $wantlv);
 
 		# Easy case, assigning to one variable, assign the whole array:
-		return $self->{VarSetFun}($self, @left, @right) if($#left == 0);
+		return $self->{VarSetFun}($self, @left, @right) if($#right <= 0);
 
 		# Assign conseq values to conseq variables. The last var gets the rest of the values.
 		# Ignore too many vars.
-		for(my $i = 0; $i <= $#right; $i++) {
-			if($i != $#right and $i == $#left) {
+		for(my $i = 0; $i <= $#left; $i++) {
+			last if($i > $#right);
+
+			if($i == $#left and $i != $#right) {
 				$self->{VarSetFun}($self, $left[$i], @right[$i ... $#right]);
 				last;
 			}
 			$self->{VarSetFun}($self, $left[$i], $right[$i]);
 		}
+
 		return @right;
 	}
 
-	# Evaluate left - may be able to avoid evaluating right:
-	my @left = &EvalTree($self, $tree->{left}, $wantlv);
+	# Flow control: if/while
+	if($oper eq 'flow') {
+		if($tree->{flow} eq 'if') {
+			# left is condition, right is body when true
+			my @left = $self->EvalTree($tree->{left}, 0);
+			return ($left[-1]) ? ($self->EvalTree($tree->{right}, 0))[-1] : 0;
+		}
+		if($tree->{flow} eq 'while') {
+			my $ret = 0; # Return val, until get something better
+			if( !$self->{PermitLoops}) {
+				$self->{PrintErrFunc}("Loops not enabled, set property PermitLoops to do so");
+				return;
+			}
+			while(1) {
+				if($self->{MaxLoopCount} && ++$self->{LoopCount} > $self->{MaxLoopCount}) {
+					$self->{PrintErrFunc}("Loop exceeded maximum iterations: MaxLoopCount = $self->{MaxLoopCount}");
+					return;
+				}
+				# left is loop condition, right is body:
+				my @left = $self->EvalTree($tree->{left}, 0);
+				return $ret unless($left[-1]);
+				$ret = ($self->EvalTree($tree->{right}, 0))[-1];
+			}
+			return $ret;
+		}
+	}
+
+	# Evaluate left - may be able to avoid evaluating right.
+	# Take care to avoid evaluating a tree twice, not just inefficient but nasty side effects with ++ & -- operators
+	my @left = $self->EvalTree($tree->{left}, $wantlv);
 	my $left = $left[$#left];
 	if(!defined($left) and $oper ne ',' and $oper ne '.') {
 		unless($self->{AutoInit}) {
@@ -476,36 +692,55 @@ sub EvalTree {
 	}
 
 	# Lazy evaluation:
-	return $left ?  &EvalTree($self, $tree->{right}{left}, $wantlv) :
-			&EvalTree($self, $tree->{right}{right}, $wantlv)		if($oper eq '?');
+	return $left ?  $self->EvalTree($tree->{right}{left}, $wantlv) :
+			$self->EvalTree($tree->{right}{right}, $wantlv)		if($oper eq '?');
 
 	# Constructing a list of variable names (for assignment):
-	return (@left, &EvalTree($self, $tree->{right}, 1))				if($oper eq ',' and $wantlv);
+	return (@left, $self->EvalTree($tree->{right}, 1))			if($oper eq ',' and $wantlv);
 
 	# More lazy evaluation:
 	if($oper eq '&&' or $oper eq '||') {
 		return 0 if($oper eq '&&' and !$left);
 		return 1 if($oper eq '||' and  $left);
 
-		my @right = &EvalTree($self, $tree->{right}, 0);
+		my @right = $self->EvalTree($tree->{right}, 0);
 
 		return($right[$#right] ? 1 : 0);
 	}
 
 	# Everything else is a binary operator, get right side - value(s):
-	my @right = &EvalTree($self, $tree->{right}, 0);
+	my @right = $self->EvalTree($tree->{right}, 0);
+	my $right = $right[-1];
 
 	return (@left, @right)	if($oper eq ',');
-#	return @right		if($oper eq ';');
+	return @right		if($oper eq ';');
 
-	# Everything else just takes a simple (non array) value, use last value in a list.
+	# Array index
+	if($oper eq '[') {
+		return undef	# Check if the array member could exist; ie have index
+			if($right !~ /^\d+$/);
+
+		return "$left\[$right]" # Return var[index] for assignment
+			if($wantlv);
+
+		return undef	# Check if the array member exists
+			if($right[-1] > @left);
+
+		return $left[$right];
+	}
+
+
+	# Everything else just takes a simple (non array) value, use last value in a list which is in $right.
 	# It is OK to concat undef.
-	my $right = $right[$#right];
 
 	if($oper eq '.') {
 		# If one side is undef, treat as empty:
 		$left = ""  unless(defined($left));
 		$right = "" unless(defined($right));
+		if(length($left) + length($right) > $self->{StringMaxLength}) {
+			$self->{PrintErrFunc}("Joined string would exceed maximum allowed %d", $self->{StringMaxLength});
+			return "";
+		}
 		return $left . $right;
 	}
 
@@ -565,6 +800,9 @@ sub EvalTree {
 	return $left == $right ? 1 : 0 if($oper eq '==');
 	return $left != $right ? 1 : 0 if($oper eq '!=');
 	return $left != $right ? 1 : 0 if($oper eq '<>');
+
+	$self->{PrintErrFunc}("Unknown operator when evaluating: '%s'", $oper);
+	return;
 }
 
 # Evaluate a function:
@@ -586,7 +824,17 @@ sub FuncValue {
 	return split $arglist[0], $arglist[$#arglist]		if($fname eq 'split');
 	return join  $arglist[0], @arglist[1 ... $#arglist]	if($fname eq 'join');
 
-	return sprintf $arglist[0], @arglist[1 ... $#arglist]	if($fname eq 'printf');
+	# Beware: could exceed max length with: printf("%2000s", "foo");
+	if($fname eq 'printf') {
+		unless($self->{EnablePrintf}) {
+			$self->{PrintErrFunc}("Function 'printf' not enabled");
+			return "";
+		}
+		my $s = sprintf $arglist[0], @arglist[1 ... $#arglist];
+		return $s if(length($s) <= $self->{StringMaxLength});
+		$self->{PrintErrFunc}("String would exceed maximum allowed %d", $self->{StringMaxLength});
+		return "";
+	}
 
 	return mktime(@arglist)					if($fname eq 'mktime');
 	return strftime($arglist[0], @arglist[1 ... $#arglist])	if($fname eq 'strftime');
@@ -628,6 +876,12 @@ sub new {
 		AutoInit	=>	0,			# If true auto initialise variables
 		ExtraFuncEval	=>	undef,			# User supplied extra function evaluator function
 		RoundNegatives	=>	0,			# Round behaves differently with -ve numbers
+		PermitLoops	=>	0,			# Are loops allowed
+		MaxLoopCount	=>	50,			# Max # all loops
+		ArrayMaxIndex	=>	100,			# Max index of an array
+		StringMaxLength	=>	1000,			# Max string length
+		EnablePrintf	=>	0,			# Enable printf function
+
 	);
 
 	my $self = bless \%ExprVars => $class;
@@ -654,7 +908,7 @@ __END__
 
 =head1 NAME
 
-Math::Expression - Evaluate arithmetic/string expressions
+Math::Expression - Safely evaluate arithmetic/string expressions
 
 =head1 SYNOPSIS
 
@@ -675,9 +929,16 @@ Math::Expression - Evaluate arithmetic/string expressions
 
 =head1 DESCRIPTION
 
-This solves the problem of evaluating expressions read from config/... files without
-the use of C<eval>.
-String and arithmetic operators are supported, as are: conditions, arrays and functions.
+Evaluating an expression from an untrusted source can result in security or denial of service attacks.
+Sometimes this needs to be done to do what the user wants.
+
+This module solves the problem of evaluating expressions read from sources such as config/...
+files and user web forms without the use of C<eval>.
+String and arithmetic operators are supported, as are: variables, loops, conditions, arrays and functions.
+
+The program may set initial values for variables and obtain their values once the expression
+has been evaluated.
+
 The name-space is managed (for security), user provided functions may be specified to set/get
 variable values.
 Error messages may be via a user provided function.
@@ -688,11 +949,15 @@ This is not designed for high computation use.
 An expression needs to be first compiled (parsed) and the resulting tree may be run (evaluated)
 many times.
 The result of an evaluation is an array.
+Variables are preserved between evaluations.
 You might also want to take computation results from stored variables.
+Method C<ParseToScalar> does it all in one: parse, check & evaluate.
+
+See examples later in this document.
 
 For further examples of use please see the test program for the module.
 
-=head2 Package functions
+=head2 Package methods
 
 =over 4
 
@@ -710,6 +975,39 @@ These options may also be given to the C<new> function.
 
 =over 4
 
+=item PermitLoops
+
+This must be set C<true> otherwise loops (C<while>) will not be allowed.
+This is to prevent a denial of service attack when the expression is from an
+untrusted source.
+
+Default: false
+
+=item MaxLoopCount
+
+This it the maximum number of times that loops will be allowed to iterate.
+Where there is more than one loop, all loops count towards this limit.
+Think carefully before making this too high.
+
+If set to zero, there is no iteration limit. This is probably unwise.
+
+The count restarts when an Eval function is used to evaluate a tree.
+
+Default: 50.
+
+=item ArrayMaxIndex
+
+The largest number that can be used as an index when assigning to an array.
+
+Default: 100.
+
+=item EnablePrintf
+
+This must be set true for the C<printf> function to be allowed.
+Beware this could take a long time to fail: C<printf('%1000000s', 'foo')>
+
+Default: 0.
+
 =item PrintErrFunc
 
 This is a printf style function that will be called in the event of an error,
@@ -719,7 +1017,9 @@ The default is C<printf STDERR>.
 =item VarHash
 
 The argument is a hash that will be used to store variables.
-If this is called several times it is possible to manage distinct name spaces.
+Changing the has between runs makes it is possible to manage distinct name spaces,
+ie different computations use different sets of variables.
+
 The name C<EmptyList> should, by convention, exist and be an empty array; this may be
 used to assign an empty value to a variable.
 
@@ -777,23 +1077,24 @@ If true automatically initialise undefined values, to the empty string or '0' de
 The default is that undefined values cause an error, except that concatentation (C<.>)
 always results in the empty string being assumed.
 
+=back
+
 Example:
 
+  my $ArithEnv = new Math::Expression(RoundNegatives => 1);
+
   my %Vars = (
 	EmptyList       =>      [()],
   );
 
-  $ArithEnv->SetOpt(VarHash => \%Vars,
+  $ArithEnv->SetOpt(
+	VarHash => \%Vars,
 	VarGetFun => \&VarValue,
 	VarIsDefFun => \&VarIsDef,
 	PrintErrFunc => \&MyPrintError,
 	AutoInit => 1,
 	);
 
-   my $ArithEnv2 = new Math::Expression(RoundNegatives => 1);
-
-=back
-
 =item ParseString
 
 This parses an expression string and returns a tree that may be evaluated later.
@@ -815,22 +1116,23 @@ This combines C<ParseString> and C<CheckTree>.
 
 =item VarSetFun
 
-This sets a variable, see the description in &SetOpt.
+This sets a variable, see the description in C<SetOpt>.
 
 =item VarSetScalar
 
-This sets a variable, see the description in &SetOpt.
+This sets a variable, see the description in C<SetOpt>.
 
 =item FuncValue
 
-This evaluates a function, see the description in &SetOpt.
+This evaluates a function, see the description in C<SetOpt>.
 
 =item EvalTree
 
-Evaluate a tree. The result is an array, if you are expecting a single value it is the last (probably $#'th) element.
+Evaluate a tree or subtree. The result is an array, if you are expecting a single value it is the last (probably $#'th) element.
 The arguments are: 0 - the value returned by C<new>; 1 - tree to evaluate; 2 - true if
 a variable name is to be returned rather than it's value (don't set this).
-You should not use this, use &Eval or &EvalToScalar instead.
+You should not use this, use methods C<Eval> or C<EvalToScalar> instead.
+This does B<not> reset the used loop count property C<LoopCount>.
 
 =item Eval
 
@@ -842,12 +1144,18 @@ The arguments are: 0 - the value returned by C<new>; 1 - tree to evaluate.
 Evaluate a tree. The result is a scalar (simple variable).
 The arguments are: 0 - the value returned by C<new>; 1 - tree to evaluate.
 
+=item ParseToScalar
+
+Parse a string, check and Evaluate its tree. The result is a scalar (simple variable).
+Undefined is returned on error.
+The arguments are: 0 - the value returned by C<new>; 1 - tree to evaluate.
+
 =back
 
 =head2 Functions that may be used in expressions
 
 The following functions may be used in expressions, if you want more than this write your own
-function evaluator and set it with &SetOpt;
+function evaluator and set it with method C<SetOpt>;
 The POSIX package is used to provide some of the functions.
 
 =over 4
@@ -877,6 +1185,7 @@ Joins arguments 1..$#, separating elements with the 0th argument.
 =item printf
 
 The standard perl C<printf>, returns the formatted result.
+To use this the option C<EnablePrintf> must be set true.
 
 =item mktime
 
@@ -889,10 +1198,12 @@ Passes all the arguments to C<strftime>, returns the result.
 =item localtime
 
 Returns the result of applying C<localtime> to the last argument.
+The variable C<_TIME> is initialised to the current time
 
 =item defined
 
 Applies the C<VarIsDefFun> to the last argument.
+Ie returns 1 if the variable is defined (has been assigned a value), 0 if it has not.
 
 =item aindex
 
@@ -900,7 +1211,7 @@ Searches the arguments for the last argument and returns the index.
 Return -1 if it is not found.
 Eg the following will return 1:
 
-  months := 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'
+  months := 'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec';
   aindex(months, 'Feb')
 
 =back
@@ -908,11 +1219,12 @@ Eg the following will return 1:
 =head2 Variables
 
 Variables can 3 three forms, there is no difference in usage between any of them.
-A variable name is either alphanumeric (starting with alpha), numeric (with
+A variable name is either alphanumeric (starting with alpha, underscore is deemed an alpha), numeric (with
 a leading C<$> or C<${>, or any non white space between C<{}> or one non
 white space after a C<$>:
 
 	Variable
+	_foo123
 	$Variable
 	${Variable}
 	$123
@@ -930,9 +1242,9 @@ matching singe or double quotes. Escapes are not looked for in literal strings.
 The operators should not surprise any Perl/C programmer, with the exception that assignemnt
 is C<:=>. Operators associate left to right except for C<:=> which associates right to left.
 Precedence may be overridden with parenthesis.
-Unary C<+> and  C<-> do not exist.
 <> is the same as C<!=>.
 
+	++ --	Pre increment/decrement only
 	+ - ~ !	(Monadic)
 	* / %
 	+ -
@@ -945,6 +1257,28 @@ Unary C<+> and  C<-> do not exist.
 	,
 	:=
 
+A semicolon (C<;>) may be used to separate statements; the value is that of the last expression.
+
+Statements may be grouped with brackes: C<{ }>
+
+	{ a := 10; b := a * 4 }
+
+=head2 Order of evaluation
+
+The order of evaluation of an expression is not defined except at sequence points.
+The sequence points are: C<while> C<if> C<;> C<?:> C<&&> C<||>.
+In particular C<&&> and C<||> only evaluate their right hand sides if they need to.
+
+Thus which element of C<a> gets updated by the code below may change in a future release:
+
+	a := (5, 6, 7); i := 0; a[++i] := ++i
+
+Multiple assignment works:
+
+	a := b := 3; if(1) a:= b := 4
+
+Sets C<a> and C<b> to C<3> and then sets them to C<4>.
+
 =head2 Arrays
 
 Variables are implemented as arrays, if a simple scalar value is wanted (eg you want to go C<+>)
@@ -974,9 +1308,22 @@ Arrays may be used to assign multiple values, eg:
 If there are too many values the last variable receives the remainder.
 If there are not enough values the last ones are unchanged.
 
-=head2 Conditions
+You may use C<[]> to numerically index into arrays to obtain and set scalar values.
+Arrays cannot contain other arrays.
+Array indexes start with C<0>.
 
-Conditional assignment may be done by use of the ternary operator:
+	a := (20,21,22); a[1] + a[2]
+	a := (20,21,22); a[1] := 9
+	i := -1; j := 2; a := (20,21,22); a[i + j] := 3
+
+When setting values you can extend an array one element at a time.
+You can create a variable by setting index C<0>.
+
+Index greater than C<ArrayMaxIndex> cannot be used to assign to an array. See method C<SetOpt>.
+
+=head2 Conditions and loops
+
+Conditional assignment B<used> to be done by use of the ternary operator, but no longer:
 
 	a > b ? ( c := 3 ) : 0
 
@@ -985,20 +1332,72 @@ is assigned a value:
 
 	a > b ? aa : bb := 124
 
+C<if> and C<while> may be used to perform conditionals and loops:
+
+	if(i < 3) { i := i + j; j := 0}
+	if(i < 3) i := 10;
+	i := 0; a := 0; if(i < 4) {i := i + 1; a := 9 }; a+i
+	i := 0; b := 1; while(++i < 4) b := b * 2;  b
+	i := 0; while(i < 4) {i := i + 1;}; i
+
+Note how the braces may be omitted if there is one statement after the C<if>.
+You can nest C<if> and C<while> within each other.
+
+If the expression is from an untrusted source, loops may cause a denial of service attack.
+So: the following are avaiable to use with C<SetOpt>: C<PermitLoops> and C<MaxLoopCount>,
+see the above description for details.
+
 =head2 Miscellaneous and examples
 
 There is no C<;> so each strings Parsed must be one expression.
+
 	my $tree1 = $ArithEnv->Parse('a := 10');
 	my $tree2 = $ArithEnv->Parse('b := 3');
 	my $tree3 = $ArithEnv->Parse('a + b');
 
 	$ArithEnv->EvalToScalar($tree1);
 	$ArithEnv->EvalToScalar($tree2);
-	print "Result: ", $ArithEnv->EvalToScalar($tree2), "\n";
+	print "Result: ", $ArithEnv->EvalToScalar($tree3), "\n";
+	say "a * b = " . $ArithEnv->ParseToScalar('a * b');
+	say $ArithEnv->ParseToScalar('"a != b is " . (a != b)');
+	say $ArithEnv->ParseToScalar('2 * 3 / 4');
+	say $ArithEnv->ParseToScalar('1.2e2 + 0');
 
 prints:
-	Result: 13
 
+	Result: 13
+	a * c = 30
+	a != b is 1
+	1.5
+	120
+
+	$ArithEnv->ParseToScalar('FirstName := "George"');
+	$ArithEnv->ParseToScalar('SurName := "Williams"');
+	say "Son's name is " . $ArithEnv->ParseToScalar('FirstName . " " . SurName');
+	say "Name is George = " . $ArithEnv->ParseToScalar('FirstName eq "George"');
+
+prints:
+
+	Son's name is George Williams
+	Name is George = 1
+
+	$ArithEnv->VarSetScalar('_TimeYesterday', time - 86400);
+	say $ArithEnv->ParseToScalar('strftime("Yesterday date=%Y/%m/%d", localtime(_TimeYesterday))');
+	say $ArithEnv->ParseToScalar('strftime("Today date=%Y/%m/%d", localtime(_TIME))');
+
+prints:
+
+	Yesterday date=2015/03/21'
+	Today date=2015/03/22'
+
+	say $ArithEnv->ParseToScalar('10 + (44, 66, 22 + 1)');
+	say $ArithEnv->ParseToScalar('c := 12; d := 3; c * d');
+
+
+prints:
+
+	33
+	36
 
 =head1 AUTHOR
 
@@ -1006,9 +1405,9 @@ Alain D D Williams <addw@phcomp.co.uk>
 
 =head2 Copyright and Version
 
-Version "1.21", this is available as: $Math::Expression::Version.
+Version "1.36", this is available as: $Math::Expression::Version.
 
-Copyright (c) 2003 Parliament Hill Computers Ltd/Alain D D Williams. All rights reserved.
+Copyright (c) 2003, 2015 Parliament Hill Computers Ltd/Alain D D Williams. All rights reserved.
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself. Please see the module source
 for the full copyright.
